<?php

namespace App\Interfaces\Repositories\Base;

use App\Interfaces\Repositories\Base\BaseAbstractRepositoryUtilities;
use App\Transports\Information\ID;

use DB;

abstract class BaseAbstractRepository extends BaseAbstractRepositoryUtilities
{
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED PROPERTIES

	/**
	 * @var array $active
	 */
	protected $active = ['active' => true];

	/**
	 * @var array $inactive
	 */
	protected $inactive = ['active' => false];

	/**
	 * @var string $table
	 * @important $table MUST be set in the final class
	 */
	protected $table = '';

	/**
	 * @var boolean $includeActive
	 * $includeActive is used to block front facing repositories from back end administration repositories that need access to inactive data
	 */
	protected $includeActive = false;

	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	
	protected function explicitField($fieldName)
	{
		return $this->table . '.' . $fieldName;
	}

	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return ($this->includeActive)
			? $this->initializeTable()->where($this->explicitField('.active'), true)
			: $this->initializeTable();
	}

	/**
	 * @method filterByFields
	 * @param array $fields
	 * @return QueryBuilder
	 */
	protected function filterByFields(array $fields = [])
	{
		return empty($fields)
			? $this->filterByActive()
			: $this->filterByActive()
				->where($fields);
	}

	/**
	 * @method filterById
	 * @param ID $id
	 * @return QueryBuilder
	 */
	protected function filterById(ID $id)
	{
		return $this->filterByFields([
			'id' => $id->id
		]);
	}

	/**
	 * @method getAutoIncrementId
	 * @param ID $id
	 * @return integer
	 * This method only exists to keep the get/extract consistent between this class and the replicated class
	 */
//	protected function getAutoIncrementId(ID $id)
//	{
//		return $this->extractAutoIncrementId($id);
//	}

	/**
	 * @method initializeTable
	 * @return QueryBuilder
	 */
	protected function initializeTable()
	{
		return DB::table($this->table);
	}

	/**
	 * @method offsetAndOrder
	 * @param integer $offset
	 * @param integer $limit
	 * @param array $orderBy
	 */
	protected function offsetAndOrder($queryBuilder, $offset = 0, $limit = 0, array $orderBy = [])
	{
		if ($offset || $limit)
		{
			$queryBuilder->skip($offset)->take($limit);
		}

		foreach ($orderBy as $field => $direction)
		{
			$queryBuilder->orderBy($field, $direction);
		}

		return $queryBuilder;
	}

	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS

	/**
	 * @method delete
	 * @param ID $id
	 * @return @TODO
	 */
	public function delete(ID $id)
	{
		return $this->filterById($id)
				->delete();
	}

	/**
	 * @method getById
	 * @param ID $id
	 * @return mixed object if record exists or null
	 */
	public function getById(ID $id)
	{
		return $this->filterById($id)
			->first();
	}

	/**
	 * @method getByName
	 * @param string $name
	 * @param 
	 * @return mixed object if record exists
	 */
	public function getByName($name)
	{
		return $this->filterByFields(['name' => $name])
			->first();
	}

	/**
	 * @method getList
	 * @param integer $offset
	 * @param integer $limit
	 * @param array $orderBy [fieldName => direction]
	 * @return Collection
	 */
	public function getList($offset = 0, $limit = 0, array $orderBy = [])
	{
		return $this->offsetAndOrder($this->filterByActive(), $offset, $limit, $orderBy)
			->get();
	}

	/**
	 * @method getTable
	 * @return string
	 */
	public function getTable()
	{
		return $this->table;
	}

	/**
	 * @method insert
	 * @param array $fields
	 * @return ID
	 */
	public function insert(Array $fields = [])
	{
		return new ID($this->initializeTable()
			->insertGetId($fields));
	}

	/**
	 * @method setActive
	 * @param ID $id
	 * @return @TODO
	 */
	public function setActive(ID $id)
	{
		return $this->update($id, $this->active);
	}

	/**
	 * @method setInactive
	 * @param ID $id
	 * @return @TODO
	 */
	public function setInactive(ID $id)
	{
		return $this->update($id, $this->inactive);
	}

	/**
	 * @method truncate
	 */
	public function truncate()
	{
		DB::table($this->getTable())->truncate();
	}

	/**
	 * @method update
	 * @param ID $id
	 * @param array fields
	 */
	public function update(ID $id, Array $fields)
	{
		return $this->filterById($id)
				->update($fields);
	}
}