<?php

namespace App\Interfaces\Repositories\Base;

use App\Interfaces\Repositories\Base\BaseAbstractRepository;
use App\Traits\Database\DatabaseSchemaHelperTraits;
use App\Transports\Information\ID;

use DB;

abstract class BaseReplicatedAbstractRepository extends BaseAbstractRepository
{
	use DatabaseSchemaHelperTraits;
	
	//################################################################################
	//################################################################################
	//################################################################################
	// CONSTANTS
	const CLUSTER_ENVIRONMENT_KEY = 'SYNCHRONIZATION_CLUSTER_ID';
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS

	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return parent::filterByActive()
			->where($this->explicitField('deleted_at'), null);
	}

	/**
	 * @method filterById
	 * @param ID $id
	 * @return QueryBuilder
	 */
	protected function filterById(ID $id)
	{
		return $this->filterByFields([
			'id'            => $id->id,
			'cluster_id'    => $id->clusterId
		]);
	}

	/**
	 * @method getCurrentClusterId
	 * @return integer
	 */
	protected function getCurrentClusterId()
	{
		return env(self::CLUSTER_ENVIRONMENT_KEY);
	}

	/**
	 * @method populateReplicationFields
	 * @param array $fields
	 * @return array
	 */
	protected function populateReplicationFields(Array $fields)
	{
		$fields['last_cluster_id']	= $this->getCurrentClusterId();
		return $fields;
	}

	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * @method delete
	 * @param ID $id
	 * @return @TODO
	 */
	public function delete(ID $id)
	{
		return $this->update($id, ['deleted_at' => $this->getCurrentTimestamp()]);
	}

	/**
	 * 
	 * @param ID $id
	 * @return mixed
	 */
	public function getById(ID $id)
	{
		return $this->filterByFields([
			'id'            => $id->id,
			'cluster_id'    => $id->clusterId
		])->first();
	}
	
	/**
	 * 
	 * @param ID $id
	 * @param string $field
	 * @return string
	 */
	public function getField(ID $id, $field)
	{
		return $this->filterById($id)
			->select($field)
			->first()
			->$field;
	}
	
	/**
	 * 
	 * @param ID $id
	 * @param array $fields
	 * @return collection
	 */
	public function getFields(ID $id, array $fields)
	{
		return $this->filterById($id)
			->select($fields)
			->first();
	}

	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$clusterId = $this->getCurrentClusterId();
		$fields = $this->populateReplicationFields($fields);
		$fields['crypto_meta'] = json_encode(encryptionConfigVersion());
		if (! isset($fields['id']))
		{
			$fields['id'] = DB::table($this->getSeriesTableName())
				->insertGetId(['cluster_id' => $clusterId]);
			$fields['cluster_id'] = $clusterId;
		}
		$this->initializeTable()
			->insert($fields);
		return new ID($fields['id'], $fields['cluster_id']);
	}

	/**
	 * @method update
	 * @param ID $id
	 * @param array fields
	 */
	public function update(ID $id, Array $fields)
	{
		$fields = $this->populateReplicationFields($fields);
		$fields['updated_at'] = $this->getCurrentTimestamp();
		return $this->filterById($id)
				->update($fields);
	}
}