<?php

namespace App\Interfaces\Repositories\Base;

use CurrentLanguage;
use DB;

abstract class TranslatedAbstractRepository
{
	/**
	 * @var ACTIVE
	 */
	protected $active = ['active' => true];

	/**
	 * @var INACTIVE
	 */
	protected $inactive = ['active' => false];

	/**
	 * @var string $baseTable
	 */
	protected $baseTable;

	/**
	 * @var string $languageTable
	 */
	protected $languageTable;

	/**
	 * @var string $databaseView
	 */
	protected $databaseView;

	/**
	 * @var string $foreignKeyName
	 */
	protected $foreignKeyName;

	/**
	 * @var boolean $includeInactive
	 */
	protected $includeInactive = false;

	/**
	 * @method getLanguageId
	 * @param $languageId
	 * @return integer
	 */
	protected function getLanguageId($languageId = false)
	{
		return ($languageId)
			? $languageId
			: CurrentLanguage::getId();
	}

	/**
	 * @method initializeTable
	 * @return QueryBuilder
	 */
	protected function initializeTable()
	{
		return DB::table($this->baseTable);
	}

	/**
	 * @method initializeTranslationTable
	 * @return QueryBuilder
	 */
	protected function initializeTranslationTable()
	{
		return DB::table($this->languageTable);
	}

	/**
	 * @method initializeView
	 * @param integer $baseId
	 * @param integer $languageId
	 */
	protected function initializeView($baseId = false, $languageId = false)
	{
		$db = DB::table($this->databaseView)
			->where('language_id', $this->getLanguageId($languageId));
		return
			($baseId)
			? $db->where('id', $baseId)
			: $db;
	}

	/**
	 * @method delete
	 * @param integer $id
	 */
	public function delete($id)
	{
		return $this->initializeTable()
			->where('id', $id)
			->delete();
	}

	/**
	 * @method deleteByName
	 * @param string $name
	 */
	public function deleteByName($name)
	{
		return $this->filterByFields([
				'name' => $name
			])->delete();
	}

	/**
	 * @method deleteTranslation
	 * @param integer $id
	 * @param integer $languageId
	 */
	public function deleteTranslation($id, $languageId = false)
	{
		return $this->initializeTranslationTable()
			->where($this->foreignKeyName, $id)
			->where('language_id', $this->getLanguageId($languageId))
			->delete();
	}

	/**
	 * @method filterBaseById
	 * @param integer $id
	 * @return QueryBuilder
	 */
	protected function filterBaseById($id)
	{
		return $this->initializeTable()
			->where('id', $id);
	}

	/**
	 * @method filterByField
	 * @param string $fieldName
	 * @param mixed $value
	 * @return QueryBuilder
	 */
	public function filterByField($fieldName, $value)
	{
		return $this->initializeTable()
			->where($fieldName, $value);
	}

	/**
	 * @method filterTranslationTableByIdLanguage
	 * @param integer $baseId
	 * @param integer $languageId
	 * @return QueryBuilder
	 */
	public function filterTranslationTableByIdLanguage($baseId, $languageId = false)
	{
		return $this->initializeTranslationTable()
			->where($this->foreignKeyName, $baseId)
			->where('language_id', $this->getLanguageId($languageId));
	}

	/**
	 * @method filterTranslationByField
	 * @param string $fieldName
	 * @param string $fieldValue
	 * @param integer $languageId
	 * @return QueryBuilder
	 */
	public function filterTranslationByField($fieldName, $fieldValue, $languageId = false)
	{
		return $this->initializeTranslationTable()
			->where('language_id', $this->getLanguageId($languageId))
			->where($fieldName, $fieldValue);
	}

	/**
	 * @method filterViewByField
	 * @param string $fieldName
	 * @param string $fieldValue
	 * @param integer $languageId
	 * @return QueryBuilder
	 */
	public function filterViewByField($fieldName, $fieldValue, $languageId = false)
	{
		return $this->initializeView($this->getLanguageId($languageId))
			->where($fieldName, $fieldValue);
	}

	/**
	 * @method filterViewById
	 * @param integer $baseId
	 * @param integer $languageId
	 * @return QueryBuilder
	 */
	public function filterViewById($baseId, $languageId = false)
	{
		return $this->filterViewByField('id', $baseId, $this->getLanguageId($languageId));
	}

	/**
	 * @method get
	 * @param integer $id
	 * @param integer $languageId
	 * @return QueryBuilder
	 */
	public function get($id, $languageId = false)
	{
		return $this->initializeView($id, $this->getLanguageId($languageId))
			->first();
	}

	/**
	 * @method getByName
	 * @param string $name
	 * @param integer $languageId
	 */
	public function getByName($name, $languageId = false)
	{
		return $this->filterViewByField(
				'base_name',
				$name,
				$this->getLanguageId($languageId)
			)
			->first();
	}

	/**
	 * @method getList
	 * @param integer $languageId
	 * @return Collection
	 */
	public function getList($languageId = false)
	{
		return $this->initializeView($this->getLanguageId($languageId))
			->get();
	}

	/**
	 * @method getRelatedTable
	 * @return string
	 */
	public function getRelatedTable()
	{
		return $this->languageTable;
	}

	/**
	 * @method getTable
	 * @return string
	 */
	public function getTable()
	{
		return $this->baseTable;
	}

	/**
	 * @method insert
	 * @param array $baseFields
	 * @param array $translatedFields
	 * @return integer auto increment id, if it exists
	 */
	public function insert(Array $baseFields, Array $translatedFields = [])
	{
		$baseId = $this->initializeTable()
			->insertGetId($baseFields);
		
		if (count($translatedFields))
		{
			$this->insertTranslation($baseId, $translatedFields);
		}

		return $baseId;
	}

	/**
	 * @method insertTranslation
	 * @param integer $id
	 * @param array $translatedFields
	 * @param boolean $returnNewId
	 * @return integer auto increment id
	 */
	public function insertTranslation($id, Array $translatedFields, $returnNewId = false)
	{
		$translatedFields[$this->foreignKeyName] = $id;
		$db = $this->initializeTranslationTable();
		return
			($returnNewId)
			? $db->insertGetId($translatedFields)
			: $db->insert($translatedFields);
	}

	/**
	 * @method setActive
	 * @param integer $id
	 */
	public function setActive($id)
	{
		return $this->update($id, $this->active);
	}

	/**
	 * @method setInactive
	 * @param integer $id
	 */
	public function setInactive($id)
	{
		return $this->update($id, $this->inactive);
	}

	/**
	 * @method setTranslationActive
	 * @param integer $id
	 * @param integer $languageId
	 */
	public function setTranslationActive($id, $languageId = false)
	{
		return $this->updateTranslation(
			$id,
			$this->active,
			$this->getLanguageId($languageId)
		);
	}

	/**
	 * @method setTranslationInactive
	 * @param integer $id
	 * @param integer $languageId
	 */
	public function setTranslationInactive($id, $languageId = false)
	{
		return $this->updateTranslation(
			$id,
			$this->inactive,
			$this->getLanguageId($languageId)
		);
	}

	/**
	 * @method update
	 * @param integer $id
	 * @param array $baseFields
	 */
	public function update($id, Array $baseFields)
	{
		return $this->filterBaseById($id)
			->update($baseFields);
	}

	/**
	 * @method updateTranslation
	 * @param integer $baseId
	 * @param array $translatedFields
	 * @param integer $languageId
	 */
	public function updateTranslation($baseId, Array $translatedFields, $languageId)
	{
		return $this->filterTranslationTableByIdLanguage(
				$baseId,
				$this->getLanguageId($languageId)
			)
			->update($translatedFields);
	}
}