<?php

namespace App\Interfaces\Repositories;

use App\Enums\Devices\DeviceStatusesEnum;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Traits\Strings\Hashes\HashGeneratorHelperTraits;
use App\Transports\Information\ID;

use DB;

class BearerUserLookupAbstractRepository extends BaseReplicatedAbstractRepository
{
	use HashGeneratorHelperTraits;

    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'bearer_tokens';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * 
	 * @method bearerHash
	 * @param string $token
	 * @return string
	 * 
	 */
	protected function bearerHash($token)
	{
		$hashAlgorithm = config('hashing.apiAuthTokenHashAlgorithm');
		return hash($hashAlgorithm, str_replace('Bearer ', '', $token));
	}
	
	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return parent::filterByActive()
			->where('device_status', '<>', DeviceStatusesEnum::DENIED)
			->orderBy('created_at', 'desc');
	}
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param string $emailHash
	 * @param string $deviceHash
	 * @param string $status
	 * @return mixed
	 */
	public function changeDeviceDetails($emailHash, $deviceHash, $description, $status)
	{
		return $this->filterByFields([
			'email_hash' => $emailHash,
			'device_hash' => $deviceHash
		])->update([
			'device_description' => $description,
			'device_status' => $status
		]);
	}
	
	/**
	 * 
	 * @param string $deviceHash
	 */
	public function confirmDevice($deviceHash)
	{
		return $this->initializeTable()
			->where(['device_hash' => $deviceHash])
			->update(['device_status' => DeviceStatusesEnum::ALLOWED]);
	}
	
	/**
	 * 
	 * @param array $emailHashes
	 * @param string $excludeDeviceHash
	 * @return recordset
	 */
	public function getDeviceHashesByEmailHashes(array $emailHashes, $excludeDeviceHash = '')
	{
		$query = $this->initializeTable()
			->select('device_hash')
			->where(function ($query) use ($emailHashes) {
				foreach ($emailHashes as $emailHash)
				{
					$query->orWhere('email_hash', '=', $emailHash);
				}
			});
		if ($excludeDeviceHash)
		{
			$query->where('device_hash', '<>', $excludeDeviceHash);
		}
		return $query->get();
	}
	
	/**
	 * 
	 * @param string $token
	 * @return string
	 */
	public function getUserLookup($token)
	{
		if ($lookup = $this->filterByFields([
			'api_token' => $this->bearerHash($token)
		])->first())
		{
			return $lookup->email_hash;
		}
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return ID
	 */
	public function insert(Array $fields = [])
	{
		$fields['api_token'] = $this->bearerHash($fields['api_token']);
		$fields['created_at'] = $this->getCurrentTimestamp();
		return parent::insert($fields);
	}
	
	/**
	 * 
	 * @param string $search
	 * @param boolean $onlyPending
	 * @return recordset
	 */
	public function searchDevices($search, $onlyPending = false)
	{
		return DB::select('call proc_search_devices(?, ?)', [$search, $onlyPending]);
	}
	
	/**
	 * 
	 * @param string $deviceHashSearch
	 * @param string $newStatus
	 * @return boolean
	 */
	public function updateDeviceStatus($deviceHashSearch, $newStatus)
	{
		return DB::select('call proc_device_status(?, ?)', [$deviceHashSearch, $newStatus]);
	}
}