<?php

namespace App\Interfaces\Repositories;

use App\Facades\Utilities\Users\Preferences;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Transports\Information\ID;

use Illuminate\Support\Collection;

use DB;

class ContactSubmissionAbstractRepository extends BaseReplicatedAbstractRepository
{
	use EncryptionHelperTraits;
	
	const ENCRYPTION_KEY = 'APP_CONTACT_FORM_ENCRYPTION_KEY';
	
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'contact_submissions';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * 
	 * @param type $record
	 * @return record
	 */
	protected function decryptRecord($record)
	{
		$stringKey = env(self::ENCRYPTION_KEY);
		
		$record->title			= $this->decryptByStringKey($record->encryted_title, $stringKey);
		$record->description	= $this->decryptByStringKey($record->encrypted_description, $stringKey);
		$record->additional		= $this->decryptByStringKey($record->encrypted_additional, $stringKey);
		unset($record->encrypted_title);
		unset($record->encrypted_description);
		unset($record->encrypted_additional);
		
		return $record;
	}
	
	/**
	 * @method initializeTable
	 * @return QueryBuilder
	 */
	protected function initializeTable()
	{
		return parent::initializeTable()
			->orderBy('created_at', 'desc');
	}
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param ID $id
	 * @return record
	 */
	public function getById(ID $id)
	{
		return $this->decryptRecord(parent::getById($id));
	}
	
	/**
	 * @return Collection of record
	 */
	public function getUnreadSubmissions()
	{
		$recordSet = $this->initializeTable()
			->where(['read' => false])
			->get();
		$return = new Collection();
		foreach ($recordSet as $record)
		{
			$return->add($this->decryptRecord($record));
		}
		
		return $return;
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$stringKey = env(self::ENCRYPTION_KEY);
				
		$fields['created_at']				= $this->getCurrentTimestamp();
		$fields['encrypted_title']			= $this->encryptByStringKey($fields['title'], $stringKey);
		$fields['encrypted_description']	= $this->encryptByStringKey($fields['description'], $stringKey);
		$fields['encrypted_additional']		= $this->encryptByStringKey($fields['additional'], $stringKey);
		unset($fields['title']);
		unset($fields['description']);
		unset($fields['additional']);
		
		return parent::insert($fields);
	}
	
	/**
	 * 
	 * @param ID $id
	 * @return boolean
	 */
	public function markAsRead(ID $id)
	{
		$result = DB::select(
			'call proc_mark_submission_as_read(?, ?)',
			[$id->clusterId, $id->id]
		);
		 return ($result) && (is_array($result)) && (array_key_exists('0', $result)) && ($result[0]->number == 1);
	}
	
	/**
	 * 
	 * @param boolean $markAsRead
	 * @return Collection
	 */
	public function unread($markAsRead = false)
	{
		$stringKey = env(self::ENCRYPTION_KEY);
		
		$submissions = DB::select('call proc_unread_messages(?)', [$markAsRead]);
		$return = new Collection();
		foreach ($submissions as $submission)
		{
			$return->push((object)[
				'id'			=> $submission->id,
				'cluster_id'	=> $submission->cluster_id,
				'created_at'	=> $submission->created_at,
				'language_code'	=> $submission->language_code,
				'type'			=> $submission->type,
				'title'			=> $this->decryptByStringKey($submission->encrypted_title, $stringKey),
				'description'	=> $this->decryptByStringKey($submission->encrypted_description, $stringKey),
				'additional'	=> $this->decryptByStringKey($submission->encrypted_additional, $stringKey),
			]);
		}
		return $return;
	}
	
	/**
	 * @return integer
	 */
	public function unreadCount()
	{
		$result = DB::select('call proc_unread_count()');
		return ($result) && (is_array($result)) && (array_key_exists('0', $result)) ? $result[0]->number : null;
	}
}