<?php

namespace App\Interfaces\Repositories;

use App\Facades\Utilities\Users\Preferences;
use App\Interfaces\Constants\PreferencesConstants;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Transports\Information\ID;

use DB;

class ContentSearchAbstractRepository  extends BaseReplicatedAbstractRepository
	implements PreferencesConstants
{
	protected $table = 'content_search_hashes';
	
	/**
	 * 
	 * @param string $channelHash
	 * @param ID $contentId
	 * @param string|null $expiresAt
	 * @param array $hashes
	 * @return void
	 */
	public function insertHashes($channelHash , ID $contentId, $expiresAt, array $hashes)
	{
		foreach ($hashes as $hash)
		{
			$this->insert([
				'channel_hash'			=> $channelHash,
				'content_id'			=> $contentId->id,
				'content_cluster_id'	=> $contentId->clusterId,
				'hash'					=> $hash,
				'expires_at'			=> $expiresAt
			]);
		}
	}
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $channelHashSeed
	 * @param array $searchTerms
	 * @param string $page
	 * @return recordset
	 * 
	 * @TODO Laravel is not escaping the search hashes so they are seen as column names
	 * The behavior happens with the select clause
	 * ->select(
				DB::raw('count(content_search_hashes.id) as weight'),
				'content.id as id',
				'content.cluster_id as cluster_id',
				'author_hash',
				'created_at',
				$this->getServerTimeField()
			)
	 * 
	 * Adding author_hash and created_at to the groupBy list resulted in properly escaped/quoted hashes in whereIn class
	 * 
	 */
	public function search($channelHash, $channelHashSeed, array $searchTerms, $page = 0)
	{
		$pageSize = Preferences::getContentPageSize();
		return DB::table('content_search_hashes')
			->select(
				DB::raw('count(content_search_hashes.id) as weight'),
				'content_id as id',
				'content_cluster_id as cluster_id',
				'content.author_hash',
				'content.created_at',
				$this->getServerTimeField()
			)
			->join('content', function ($join){
				$join->on('content.id', '=', $this->table . '.content_id')
					->on('content.cluster_id', '=', $this->table . '.content_cluster_id');
			})
			->groupBy('content_id', 'content_cluster_id', 'author_hash', 'created_at')
			->where('content_search_hashes.channel_hash', $channelHash)
			->where('content.active', true)
			->where($this->table . '.active', true)
			->whereNull($this->table . '.deleted_at')
			->whereNull('content.deleted_at')
			->where(function ($query) use ($channelHashSeed, $searchTerms){
				foreach ($searchTerms as $term)
				{
					$query->orWhereRaw('hash = sha2(concat(?, ?, hash_seed), ?)',
						[$term, $channelHashSeed, config('hashing.genericHashLength')]
					);
				}
			})
			->orderBy('weight', 'desc')
			->orderBy('created_at', 'desc')
			->take($pageSize)
			->skip($page * $pageSize)
			->get();
	}
}
