<?php

namespace App\Interfaces\Utilities\Files;

use App\Interfaces\Utilities\Files\FileIOOpenModesEnum;

abstract class FileIOUtilityAbstractInterface
{
	/**
	 * @var mixed $currentPath
	 */
	protected $currentPath;

	/**
	 * @var integer $handle
	 */
	protected $handle = false;

	/**
	 * @method getIoMethod
	 * @param string $method
	 */
	protected function getIoMethod($method)
	{
		return $method . FileIOOpenModesEnum::binaryMode;
	}

	/**
	 * @method getPath
	 * @param mixed $path
	 * @return string
	 */
	protected function getPath($path)
	{
		return is_array($path)
			? implode(DIRECTORY_SEPARATOR, $path)
			: $path;
	}

	/**
	 * @method partialPath
	 * @param mixed $path
	 * @param integer $numberSegments
	 * @return string
	 */
	protected function partialPath($path, $numberSegments = 1)
	{
		$pathArray = is_array($path)
			? $path
			: explode(
				strpos($path, '/') ? '/' : '\\',
				$path
			);
		return $this->getPath(
			array_slice($pathArray, 0, $numberSegments)
		);
	}

	/**
	 * @method close
	 */
	public function close()
	{
		if ($this->handle) fclose($this->handle);
		$this->currentPath = $this->handle = false;
	}

	/**
	 * @method delete
	 * @param mixed $path
	 * @return boolean
	 */
	public function delete($path)
	{
		return $this->exists($path)
			? unlink($this->getPath($path))
			: false;
	}

	/**
	 * @method exists
	 * @param mixed $path
	 * @return boolean
	 */
	public function exists($path)
	{
		return file_exists($this->getPath($path));
	}

	/**
	 * @method fileSize
	 * @param mixed $path
	 * @return integer
	 */
	public function fileSize($path)
	{
		return filesize($this->getPath($path));
	}

	/**
	 * @method makeFolder
	 * @param mixed $path
	 * @return boolean
	 */
	public function makeFolder($path)
	{
		if (! $this->exists($path))
		{
			return mkdir(
				$this->getPath($path),
				$this->permissions($this->partialPath($path)),
				true
			);
		}
		return false;
	}

	/**
	 * @method open
	 * @param mixed $path
	 * @param string $ioMethod
	 * @return boolean
	 */
	public function open(
		$path,
		$ioMethod = FileIOOpenModesEnum::writeMode
	)
	{
		$this->currentPath = $path;

		return boolval(
			$this->handle = fopen(
				$this->getPath($path),
				$ioMethod
			)
		);
	}

	/**
	 * @method permissions
	 * @param mixed $path
	 * @return integer
	 */
	public function permissions($path)
	{
		return fileperms($this->getPath($path));
	}

	/**
	 * @method read
	 * @param integer $length
	 * @return string
	 */
	public function read($length = false)
	{
		if ($fileLength = ($length)
			? $length
			: $this->fileSize($this->currentPath))
		{
			return ($length)
				? fread($this->handle, $length)
				: fread($this->handle, $this->fileSize($this->currentPath));
		}
	}

	/**
	 * @method readAll
	 * @param mixed $path
	 * @return string
	 */
	public function readAll($path)
	{
		return file_get_contents(
			$this->getPath($path)
		);
	}

	/**
	 * @method write
	 * @param string $content
	 * @param integer $length
	 */
	public function write($content, $length = false)
	{
		return fwrite(
			$this->handle,
			$content,
			($length)
				? $length
				: strlen($content)
		);
	}

	/**
	 * @method writeAll
	 * @param mixed $path
	 * @param string $content
	 * @return boolean
	 */
	public function writeAll(
		$path,
		$content
	)
	{
		return file_put_contents(
			$this->getPath($path),
			$content
		);
	}
}