<?php

namespace App\Traits\Strings\Encryption;

use App\Enums\Encryption\StringEncryptionAlgorithmsEnum;
use App\Transports\Encryption\EncryptedInfo\PagedEncryption;
//use App\Transports\Encryption\EncryptedInfo\PublicPrivate\DataWithKeys;
use App\Traits\Strings\Utilities\StringUtilityTraits;

/**
 * Description of EncryptionHelperTraits
 *
 * @author Scott Kerns
 */
trait EncryptionHelperTraits {
	use StringUtilityTraits;
	
	
	// ######################################################################
	// PROTECTED METHODS
	
	/**
	 * 
	 * @param string $algorithmName
	 * @return array
	 */
	protected function getKeyPairAlgorithmConfig($algorithmName = '')
	{
		$configName = ($algorithmName) ? $algorithmName : $this->getKeyPairAlgorithmConfigName();
		return config("encryption.publicPrivateKeys.$configName");
	}
	
	
	protected function getKeyPairAlgorithmConfigName()
	{
		return config('encryption.publicPrivateKeys.algorithm');
	}
	
	/**
	 * 
	 * @param type $stringKeyAlgorithm
	 * @return string
	 */
	protected function getStringAlgorithmConfigName($stringKeyAlgorithm = StringEncryptionAlgorithmsEnum::DEFAULT)
	{
		return ($stringKeyAlgorithm == StringEncryptionAlgorithmsEnum::DEFAULT)
			? config('encryption.stringKeys.algorithm')
			: $stringKeyAlgorithm;
	}
	
	/**
	 * 
	 * @param string $settingName
	 * @param string $stringKeyAlgorithm
	 * @return string
	 */
	protected function getStringKeyAlgorithmConfig($settingName = '', $stringKeyAlgorithm = StringEncryptionAlgorithmsEnum::DEFAULT)
	{
		$configPathArray = [
			'encryption',
			'stringKeys',
			$this->getStringAlgorithmConfigName($stringKeyAlgorithm)
		];
		if ($settingName)
		{
			$configPathArray[] = $settingName;
		}
		return config(implode('.', $configPathArray));
	}
	
	/**
	 * 
	 * @param type $algorithmName
	 * @return boolean
	 */
	protected function isValidStringAlgorithm($algorithmName)
	{
		return in_array($algorithmName, config('encryption.stringKeys.valid-algorithms'));
	}
	
	/**
	 * 
	 * @param string $iv
	 * @param string $encryptedKey
	 * @param string $encryptedString
	 * @param string $algorithm
	 */
	protected function packsealWithPublicKey($iv, $encryptedKey, $encryptedString, $algorithm = '')
	{
		return implode(
			';',
			[
				'alg:' . (($algorithm) ? $algorithm : config('encryption.publicPrivateKeys.algorithm')),
				'iv:' . base64_encode($iv),
				'key:' . base64_encode(is_array($encryptedKey) ? $encryptedKey[0] : $encryptedKey),
				'str:' . base64_encode($encryptedString)
			]
		);
	}
	
	/**
	 * 
	 * @param string $encryptedString
	 * @param string $algorithm
	 * @return string
	 */
	protected function packEncryptedWithPublicKey($encryptedString, $algorithm)
	{
		return implode(
			';',
			[
				'alg:' . $algorithm,
				'str:' . base64_encode($encryptedString)
			]
		);
	}
	
	/**
	 * 
	 * @param string $name
	 * @param string $string
	 * @return string
	 */
	protected function removePackSectionName($name, $string)
	{
		return substr($string, strlen($name) + 1);
	}
	
	/**
	 * 
	 * @param string $encryptedString
	 * @return object
	 */
	protected function unpackPublicKeyEncrypted($packedEncryptedString)
	{
//		list($alg, $str) = explode(';', $packedEncryptedString);
//		return new DataWithKeys(
//			base64_decode($this->removePackSectionName('alg', $alg)),
//			base64_decode($this->removePackSectionName('str', $str))
//		);
		$sections = explode(';', $packedEncryptedString);
		$return = [];
		foreach ($sections as $section)
		{
			list($sectionName, $sectionData) = explode(':', $section);
			$return[$sectionName] = $sectionData;
		}
		return (object)$return;
	}
	
	
	// ######################################################################
	// ######################################################################
	// ######################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param string $encryptedString
	 * @param string $privateKey
	 * @param boolean $serialize
	 * @return mixed
	 */
	public function decryptByPrivateKey(
		$targetString,
		$privateKey,
		$serialize = false
	)
	{
		$unpackedData = $this->unpackPublicKeyEncrypted($targetString);
		$algorithmConfig = $this->getKeyPairAlgorithmConfig($unpackedData->alg);
		openssl_private_decrypt(
			$unpackedData->str,
			$decryptedString,
			$privateKey,
			$algorithmConfig['encryption']['options']
		);
		return ($serialize)
			? $this->deSerialize($decryptedString)
			: $decryptedString;
	}
	
	/**
	 * 
	 * @param string $encryptedString
	 * @param string $key
	 * @param boolean $serialize
	 * @return mixed
	 */
	public function decryptByStringKey(
		$encryptedString,
		$key,
		$serialize = false
	)
	{
		$payload = $this->unpackEncryptByStringKeyPayload($encryptedString);
		if ($payload)
		{
			$algorithmConfigName = $payload->alg;
			if ($this->isValidStringAlgorithm($algorithmConfigName))
			{
				$cipherName = $this->getStringKeyAlgorithmConfig('opensslCipherName', $algorithmConfigName);
				$cipherOptions = $this->getStringKeyAlgorithmConfig('opensslOptions', $algorithmConfigName);
//				$decryptedString = trim(openssl_decrypt(
//						$payload->str,
//						$cipherName,
//						base64_decode($key),
//						$cipherOptions,
//						base64_decode($payload->iv)
//					),
//					chr(0)
//				);
				$decryptedString = openssl_decrypt(
					$payload->str,
					$cipherName,
					base64_decode($key),
					$cipherOptions,
					base64_decode($payload->iv)
				);
				return ($serialize)
					? $this->deSerialize($decryptedString)
					: $decryptedString;
			}
		}
	}
	
		/**
	 * 
	 * @param mixed $targetToEncrypt
	 * @param string $publicKey
	 * @param integer $timestamp
	 * @param boolean $serialize
	 * @return string
	 */
	public function encryptByPublicKey($targetToEncrypt, $publicKey, $timestamp = 0, $serialize = false)
	{
		$stringToEncrypt = ($serialize)
			? $this->serialize($targetToEncrypt)
			: $targetToEncrypt;
		$algorithmName = $this->getKeyPairAlgorithmConfigName();
		$algorithmConfig = $this->getKeyPairAlgorithmConfig($algorithmName);
		openssl_public_encrypt(
			$stringToEncrypt,
			$encryptedString,
			$publicKey,
			$algorithmConfig['encryption']['options']
		);
		return $this->packEncryptedWithPublicKey($encryptedString, $algorithmName);
	}
	
	/**
	 * 
	 * @param mixed $targetToEncrypt
	 * @param string $key
	 * @param integer $timestamp
	 * @param boolean $serialize
	 * @return string
	 */
	public function encryptByStringKey(
		$targetToEncrypt,
		$key,
		$timestamp = 0,
		$serialize = false,
		$stringKeyAlgorithm = StringEncryptionAlgorithmsEnum::DEFAULT
	)
	{
		$algorithmName = $this->getStringAlgorithmConfigName($stringKeyAlgorithm);
		$timestamp = ($timestamp) ? $timestamp : timestamp();
		$stringToEncrypt = ($serialize) ? json_encode($targetToEncrypt) : $targetToEncrypt;
		$iv = $this->generateStringKeyIv($timestamp, $algorithmName);
        $encrypted = openssl_encrypt(
//			$this->padMessage($stringToEncrypt, $this->getStringKeyAlgorithmConfig('paddingLengthBytes', $algorithmName)),
			$stringToEncrypt,
			$this->getStringKeyAlgorithmConfig('opensslCipherName', $algorithmName),
			base64_decode($key),
			$this->getStringKeyAlgorithmConfig('opensslOptions', $algorithmName),
			$iv
		);
		$encryptedAndPackaged = $this->packEncryptByStringPayload(
			$encrypted,
			$iv,
			$timestamp,
			$algorithmName
		);
		return $encryptedAndPackaged;
	}
	
	/**
	 * 
	 * @param string $algorithmName
	 * @return string
	 */
	public function generateKey($algorithmName = StringEncryptionAlgorithmsEnum::DEFAULT)
	{
		$algorithm = config('encryption.stringKeys.algorithm');
		$keyLength = config("encryption.stringKeys.$algorithm.algorithmMaxKeyLength");
		return $this->generateRandomString($keyLength, $keyLength, true);
	}
	
	/**
	 * 
	 * @param string $algorithmName
	 * @return string
	 */
	public function generateStringKey($algorithmName = StringEncryptionAlgorithmsEnum::DEFAULT)
	{
		return $this->generateKey($algorithmName);
	}
	
	/**
	 * 
	 * @param integer $timestamp
	 * @param string $algorithmName
	 * @return string
	 */
	public function generateStringKeyIv($timestamp, $algorithmName)
	{
		$hash = hash(config('hashing.genericHashDriver'), $timestamp);
		return substr($hash, 0, config("encryption.stringKeys.$algorithmName.algorithmIvLength"));
	}
	
	/**
	 * 
	 * @return string
	 */
	public function getPrivateKey()
	{
		return file_get_contents(env('APP_ENCRYPTION_PRIVATE_KEY_PATH'));
	}
	
	/**
	 * 
	 * @return string
	 */
	public function getPublicKey()
	{
		return file_get_contents(env('APP_ENCRYPTION_PUBLIC_KEY_PATH'));
	}
	
	/**
	 * 
	 * @param string $message
	 * @param integer $blockSize
	 */
//	public function padMessage($message, $blockSize)
//	{
//		$numberPadded = $blockSize - (strlen($message) % $blockSize);
//		$padded = $message . str_repeat(chr(0), $numberPadded);
//		return $padded;
//	}

	/**
	 * 
	 * @param string $encryptedString
	 * @param string $iv
	 * @param string $timestamp
	 * @param string $algorithmName
	 * @return string
	 */
	public function packEncryptByStringPayload($encryptedString, $iv, $timestamp, $algorithmName)
	{
		return implode(';', [
			'alg:' . $algorithmName,
			'iv:' . base64_encode($iv),
			'str:' . $encryptedString,
			'ts:' . $timestamp
		]);
	}
	
	/**
	 * 
	 * @param mixed $targetToEncrypt
	 * @param string $publicKey
	 * @param boolean $serialize
	 * @return string
	 */
	public function sealWithPublicKey($targetToEncrypt, $publicKey, $serialize = false)
	{
		$stringToEncrypt = ($serialize)
			? $this->serialize($targetToEncrypt)
			: $targetToEncrypt;
		$algorithm = config('encryption.publicPrivateKeys.algorithm');
		$ivLength = config("encryption.publicPrivateKeys.$algorithm.encryption.ivLength");
		$iv = $this->generateRandomString($ivLength, $ivLength);
		openssl_seal(
			$stringToEncrypt,
			$encryptedString,
			$encryptedKeys,
			[$publicKey],
			config("encryption.publicPrivateKeys.$algorithm.encryption.algorithm"),
			$iv
		);
		return $this->packsealWithPublicKey($iv, $encryptedKeys, $encryptedString);
	}
	
	/**
	 * 
	 * @param string $target
	 * @return object
	 */
	public function unpackEncryptByStringKeyPayload($target)
	{
		if (! $target){return null;}
		$sections = explode(';', $target);
		foreach ($sections as $section)
		{
			list($name, $value) = explode(':', $section);
			$payloadArray[$name] = $value;
		}
		return (object)$payloadArray;
	}
	
	/**
	 * 
	 * @param string $encryptedString
	 * @param string $privateKey
	 * @param boolean $serialize
	 * @return mixed
	 */
	public function unsealWithPrivateKey(
		$targetString,
		$privateKey,
		$serialize = false,
		$stringKeyAlgorithm = StringEncryptionAlgorithmsEnum::DEFAULT
	)
	{
		$unpackedData = $this->unpackPublicKeyEncrypted($targetString);
		$algorithm = $unpackedData->alg;
		openssl_open(
			base64_decode($unpackedData->str),
			$decryptedString,
			base64_decode($unpackedData->key),
			$privateKey,
			config("encryption.publicPrivateKeys.$algorithm.encryption.algorithm"),
			base64_decode($unpackedData->iv)
		);
		return ($serialize)
			? $this->deSerialize($decryptedString)
			: $decryptedString;
	}
}
