import definitions from './definitions.js';

const JsonToMarkup = function ()
{
	const thisProcessor = this;

	/**
	 * 
	 * @param {string} commandName 
	 * @param {object} nodeAttributes 
	 * @param {object | undefined} attributeToOptionMap 
	 * @returns string
	 */
	const buildCommand = function (commandName, nodeAttributes, attributeToOptionMap) {
		return ['=', commandName,
			'(', buildOptions(nodeAttributes, attributeToOptionMap), ')'
		].join('');
	}

	/**
	 * 
	 * @param {object} nodeAttributes 
	 * @param {object} attributeToOptionMap dom attribute to option attribute name mapping
	 * @returns string
	 */
	const buildOptions = function (nodeAttributes, attributeToOptionMap) {
		const getOptionName = function (attributeName) {
			if (attributeName == 'default') {return attributeName;}
			return (attributeToOptionMap && (attributeName in attributeToOptionMap)) ? attributeToOptionMap[attributeName] : attributeName;
		}
		const options = [];
		for (let attributeName in nodeAttributes) {
			if ((attributeName == 'default') || (attributeName in attributeToOptionMap)) {
				options.push(
					getOptionName(attributeName) + '="' + nodeAttributes[attributeName] + '"'
				);
			}
		}
		return options.join(' ');
	}

	/**
	 * 
	 * @param {string} tokenName
	 * @returns *
	 */
	const getJsonNodeInfo = function (tokenName) {
		return _.find(definitions.topLevelCommands, ((def) => {
			return def.token == tokenName;
		})).jsonNode;
	}

	/**
	 * 
	 * @param {array} markArray 
	 */
	const getPriorityMark = function (markArray) {
		const markPriorities = {link: 4, underline: 3, italic: 2, bold: 1};
		let currentMarkType = '', currentMarkIndex = 0;
		markArray.forEach((mark, markIndex) => {
			if ((mark.type in markPriorities) && (markPriorities[mark.type] > markPriorities[currentMarkType])) {
				currentMarkType = mark.type;
				currentMarkIndex = markIndex;
			}
		});
		return markArray[currentMarkIndex];
	}

	/**
	 * 
	 * @param {*} node 
	 * @returns string
	 */
	const processNode = function (node) {
		let returnSource = '';
		if (Array.isArray(node)) {
			node.forEach((childNode) => {
				returnSource += processNode(childNode);
			});
		} else if (typeof node === 'object') {
			switch (node.type) {
				case 'image':
					returnSource += buildCommand('Image', node.attrs, getJsonNodeInfo('Image').attributeMap);
					break;
				case 'text':
					if (Array.isArray(node.marks) && node.marks.length) {
						const topMark = getPriorityMark(node.marks);
						switch (topMark.type) {
							case 'bold':
								returnSource += buildCommand('Bold', {default: node.text});
								break;
							case 'italic':
								returnSource += buildCommand('Italic', {default: node.text});
								break;
							case 'link':
								returnSource += buildCommand(
									'Link',
									{link: topMark.attrs.href, text: node.text},
									getJsonNodeInfo('Link').attributeMap
								);
								break;
							case 'underline':
								returnSource += buildCommand('Underline', {default: node.text});
								break;
							default:
								returnSource += node.text;
						}
					} else {
						returnSource += node.text;
					}
					break;
				default: // paragraph
					if (Array.isArray(node.content)) {
						returnSource += processNode(node.content) + '\n';
					} else {
						returnSource += '\n';
					}
			}
		}
		return returnSource;
	}

	/**
	 * @param {object} jsonObject
	 * @returns string
	 * top level node is the doc
	 */
	thisProcessor.convertToCode = function (jsonObject) {
		return processNode(jsonObject.content);
	}
}

export default JsonToMarkup