import definitions from './definitions.js';
import Node from './node.js';

const availableCommands = definitions.availableCommands;

const Tokenizer = function (content)
{
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED PROPERTIES

	let currentStringLocation = 0,
		EOF = false,
		stateStack = [];

	const thisTokenizer = this,
		commandPattern = /^=[A-Z]{1}[a-zA-Z]{0,20}\(/,
		newLine = "\n";

	//################################################################################
	//################################################################################
	//################################################################################
	// PRIVATE METHODS

	/**
	 * 
	 * @param {string} workingString 
	 * @returns object | undefined
	 */
	const isAvailableCommandCheck = function (workingString) {
		const commandMatch = workingString.match(commandPattern);
		if (commandMatch)
		{
			const commandCode = commandMatch[0];
			const commandToken = definitions.tokenConversions.codeToCommand(commandCode);
			const index = _.findIndex(availableCommands, (availableCommand) => {
				return availableCommand.token == commandToken;
			});
			return (-1 === index) ? null : availableCommands[index];
		}
	}

	/**
	 * 
	 * @param {Node} workingNode
	 * parses content looking for options
	 */
	const setOptions = function (workingNode) {
		let inQuote = false, inAttributeValue = false, attributeName = '', accumulate = '', currentCharacter = '';
		while (! thisTokenizer.EOF())
		{
			currentCharacter = content.substr(currentStringLocation, 1);
			switch (currentCharacter)
			{
				case '"':
					inQuote = ! inQuote;
					break;
				case '=':
					if (! inQuote)
					{
						inAttributeValue = true;
						attributeName = accumulate;
						accumulate = '';
					}
					break;
				case ' ':
				case ',':
					if (inQuote) {
						accumulate += currentCharacter;
					} else {
						if (inAttributeValue && accumulate) {
							workingNode.addOption(accumulate, attributeName);
							attributeName = accumulate = '';
							inAttributeValue = false;
						}
					}
					break;
				case '\n':
					if (accumulate) {
						workingNode.addOption(accumulate, attributeName);
					}
					return;
				case ')':
					if (inQuote) {
						accumulate += currentCharacter;
					} else {
						if (accumulate) {
							workingNode.addOption(accumulate, attributeName);
						}
						currentStringLocation++;
						return;
					}
					break;
				default:
					accumulate += currentCharacter;
			}
			currentStringLocation++;
		}
		if (accumulate) {
			workingNode.addOption(accumulate, attributeName);
		}
	}

	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS

	/**
	 * @returns boolean
	 */
	thisTokenizer.EOF = function () {
		return currentStringLocation >= content.length;
	}

	/**
	 * @modifies currentStringLocation
	 * @returns Node
	 */
	thisTokenizer.next = function () {
		let accumulate = '', nodeType = '', workingString = '';
		while (! thisTokenizer.EOF()) {
			workingString = content.substr(currentStringLocation);
			if (workingString.substr(0, newLine.length) === newLine) {
				if (accumulate) {
					return new Node('content', accumulate);
				}
				currentStringLocation++;
				return new Node('NewLine');
			}
			// let commandMatch = workingString.match(commandPattern);
			// if (commandMatch) {
			// 	if (accumulate) {
			// 		return new Node('content', accumulate);
			// 	}
			// 	let commandCode = commandMatch[0];
			// 	currentStringLocation += commandCode.length;
			// 	const workingNode = new Node(definitions.tokenConversions.codeToCommand(commandCode), commandCode);
			// 	setOptions(workingNode);
			// 	return workingNode;
			// }
			let command = isAvailableCommandCheck(workingString);
			if (command) {
				if (accumulate) {
					return new Node('content', accumulate);
				}
				currentStringLocation += definitions.tokenConversions.commandToCode(command.token).length;
				const workingNode = new Node(command.handler);
				setOptions(workingNode);
				return workingNode;
			}
			accumulate += workingString.substr(0, 1);
			currentStringLocation++;
		}
		return new Node('content', accumulate);
	}
}

export default Tokenizer