import DomBuilder from './dom-builder.js';
import JsonToMarkup from './json2markup.js';
import Tokenizer from './text-editor-tokenizer.js';

import { Editor, Mark } from '@tiptap/core';
import Image from '@tiptap/extension-image';
import Link from '@tiptap/extension-link';
import StarterKit from '@tiptap/starter-kit';
import Underline from '@tiptap/extension-underline';

const marks = {
	bold: {command: 'unsetBold', 'replace': true},
	italic: {command: 'unsetItalic', 'replace': true},
	link: {command: 'unsetLink', 'replace': false},
	underline: {command: 'unsetUnderline', 'replace': true}
};

export default {
	data() {
		return {
			iconIdEmojis: this.$utilities.strings.uniqueStrings.generate('text-editor-emojis'),
			iconIdImages: this.$utilities.strings.uniqueStrings.generate('text-editor-images'),
			iconIdLinks: this.$utilities.strings.uniqueStrings.generate('text-editor-links'),
			jsonToMarkup: new JsonToMarkup,
			wysiwygEditor: null
		}
	},

	props: {
		content: {
			default: '',
			required: false,
			type: String
		},
		toolbarIconsTabIndex: {
			default: -1,
			required: false,
			type: Number
		}
	},

	methods: {
		characterSelected(characterEvent) {
			const character = document.createElement('span');
			character.innerHTML = characterEvent.options.code;
			this.wysiwygEditor.commands.insertContent(character.innerHTML);
		},

		getParsedContent(content) {
			const thisRenderer = this,
				domContainer = document.createElement('div'),
				tokenizer = new Tokenizer(content),
				domBuilder = new DomBuilder(domContainer);
			while (! tokenizer.EOF())
			{
				let node = tokenizer.next();
				domBuilder.handle(node);
			}
			return domContainer.innerHTML;
		},

		imageSelected(imageEvent) {
			this.wysiwygEditor.commands.setImage({
				src: imageEvent.options.link,
				alt: imageEvent.options.alt
			});
		},

		linkSelected(linkEvent) {
			this.wysiwygEditor.commands.toggleLink({
				href: linkEvent.options.link,
				target: "_blank"
			});
		},

		reset() {
			this.setContent('');
		},

		setContent(content) {
			this.wysiwygEditor.commands.setContent(this.getParsedContent(content));
		},

		toolbarClicked() { },

		/**
		 * 
		 * @param {string} doNotChange 
		 * @returns string
		 */
		unsetOtherMarks(doNotChange) {
			for (let markName in marks) {
				let currentMark = marks[markName];
				if ((markName !== doNotChange) && (currentMark.replace)) {
					this.wysiwygEditor.commands[currentMark.command]();
				}
			}
		},

		updateContent(jsonContent) {
			const newContent = this.jsonToMarkup.convertToCode(jsonContent);
			this.$emit('input', newContent);
			this.$emit('change', newContent);
		}
	},

	mounted() {
		const thisWysiwyg = this;
		this.wysiwygEditor = new Editor({
			content: this.getParsedContent(this.content),
			element: $(this.$el).find('[data-wysiwyg]')[0],
			extensions: [
				// Emoji,
				Image,
				Link,
				StarterKit.configure({history: false}),
				Underline
			],
			onUpdate({editor}) {
				thisWysiwyg.updateContent(editor.getJSON());
			}
		});
	}
}